package UI.GameState;

import UI.GameFrame;
import UI.MyListener;

import java.awt.event.*;

public class MyTank extends TankState {

    private boolean keyUP, keyDOWN, keyRIGHT, keyLEFT, keySpace;
    private boolean mousePress;
    private int mouseX, mouseY;
    private MyListener myListener = new MyListener();
    private KeyHandler1 keyHandler1 = new KeyHandler1();
    private MouseHandler mouseHandler;


    public MyTank(WallState walls) {
        super(walls);
        myListener.add(keyHandler1);

        keyUP = false;
        keyDOWN = false;
        keyRIGHT = false;
        keyLEFT = false;
        keySpace = false;

        mousePress = false;
        mouseX = 0;
        mouseY = 0;

        mouseHandler = new MouseHandler();
    }

    /**
     * The method which updates the game state.
     */
    public void update() {
        //
        // Update the state of all game elements
        //  based on user input and elapsed time ...
        //

        int lastx = locX,lasty = locY,lastr = rotate;

        if (mousePress) {
            locY = mouseY - diam / 2;
            locX = mouseX - diam / 2;
        }
        if (keyUP) {
            if (rotate == 0)
                rotate = 360;
            locY -= 8;
            if ((rotate > 180 && rotate < 270)|| rotate ==180 )
                rotate +=10;
            else if ( (rotate > 270 && rotate < 360) || rotate == 360)
                rotate -=10;

            else
                rotate =270;
        }
        if (keyDOWN) {
            rotate = rotate % 360;
            locY += 8;
            if ( (rotate  > 0 && rotate < 90 )|| rotate == 0)
                rotate +=10;
            else if ( (rotate > 90 && rotate <180 )|| rotate == 180 )
                rotate -=10;
            else
                rotate = 90;
        }
        if (keyLEFT ){
            rotate = rotate % 360;
            locX -= 8;
            if ( (rotate> 90 && rotate <180) || rotate== 90 )
                rotate +=10;
            else if ( (rotate >180 && rotate < 270) || rotate ==270 )
                rotate -=10;
            else
                rotate =180;
        }
        if (keyRIGHT ) {
            rotate = rotate % 360;
            locX += 8;
            if ( (rotate > 270 && rotate < 360) || rotate ==270 )
                rotate +=10;
            else if ( (rotate < 90 && rotate > 0) || rotate == 90 )
                rotate -=10;
            else
                rotate = 0;
        }
        if (keySpace){
            if (bullets.size() == 0 || System.currentTimeMillis() -bullets.get(bullets.size()-1).getCreateTime()>500 ) {
                BulletState newBullet = new BulletState(walls, this);
                bullets.add(newBullet);
            }
        }


        if (walls.isWallForTank(locX,locY)){
            locX = lastx;
            locY = lasty;
        }

        locX = Math.max(locX, 0);
        locX = Math.min(locX, GameFrame.GAME_WIDTH - diam);
        locY = Math.max(locY, 0);
        locY = Math.min(locY, GameFrame.GAME_HEIGHT - diam);

    }

    public MouseListener getMouseListener() {
        return mouseHandler;
    }
    public MouseMotionListener getMouseMotionListener() {
        return mouseHandler;
    }
    public MyListener getMyListener() {
        return myListener;
    }


    /**
     * The keyboard handler.
     */
    class KeyHandler1 extends KeyAdapter {

        @Override
        public void keyPressed(KeyEvent e) {

            switch (e.getKeyCode()) {
                case KeyEvent.VK_UP:
                    keyUP = true;
                    break;
                case KeyEvent.VK_DOWN:
                    keyDOWN = true;
                    break;
                case KeyEvent.VK_LEFT:
                    keyLEFT = true;
                    break;
                case KeyEvent.VK_RIGHT:
                    keyRIGHT = true;
                    break;
                case KeyEvent.VK_SPACE:
                    keySpace = true;
                    break;
                case KeyEvent.VK_ESCAPE:
                    gameOver = true;
                    break;
            }
        }

        @Override
        public void keyReleased(KeyEvent e) {
            switch (e.getKeyCode())
            {
                case KeyEvent.VK_UP:
                    keyUP = false;
                    break;
                case KeyEvent.VK_DOWN:
                    keyDOWN = false;
                    break;
                case KeyEvent.VK_LEFT:
                    keyLEFT = false;
                    break;
                case KeyEvent.VK_RIGHT:
                    keyRIGHT = false;
                    break;
                case KeyEvent.VK_SPACE:
                    keySpace = false;
                    break;
            }
        }

    }


    /**
     * The mouse handler.
     */
    class MouseHandler extends MouseAdapter {

        @Override
        public void mousePressed(MouseEvent e) {
            mouseX = e.getX();
            mouseY = e.getY();
            mousePress = true;
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            mousePress = false;
        }

        @Override
        public void mouseDragged(MouseEvent e) {
            mouseX = e.getX();
            mouseY = e.getY();
        }
    }
}
