package UI;
import UI.GameState.BulletState;
import UI.GameState.EnemyTank;
import UI.GameState.MyTank;
import UI.GameState.TankState;
import UI.LogIn.Setting;
import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

public class Rendering {

    private Image image1;
    private Image image2;
    public static final int GAME_HEIGHT = 720;                  // 720p game resolution
    public static final int GAME_WIDTH = 16 * GAME_HEIGHT / 9;  // wide aspect ratio
    int lastRender = -1;
    private ArrayList<Float> fpsHistory;
    private Setting setting = new Setting();

    public Rendering(){
        fpsHistory = new ArrayList<>(100);
    }

    /**
     * Rendering all game elements based on the game state.
     */
    // Draw all game elements according
    //  to the game 'state' using 'g2d' ...
    public void doRendering(Graphics2D g2d, MyTank myTank, EnemyTank enemyTank, ArrayList<BulletState> bullets) {
        // Draw background
        myTank.getWalls().paintComponent(g2d);
        enemyTank.getWalls().paintComponent(g2d);

        // Draw my tank;
        try {
            image1 = ImageIO.read(new File(setting.getTankShapePath()));
        } catch (IOException e) {
            System.out.println(e);
        }
// Rotation information
        double rotationRequired = Math.toRadians (myTank.rotate);
        BufferedImage image11 = (BufferedImage)image1;
        double locationX = image11.getWidth() / 2;
        double locationY = image11.getHeight() / 2;
        AffineTransform tx = AffineTransform.getRotateInstance(rotationRequired, locationX, locationY);
        AffineTransformOp op = new AffineTransformOp(tx, AffineTransformOp.TYPE_BILINEAR);
// Drawing the rotated image at the required drawing locations
        g2d.drawImage(op.filter( image11, null),myTank.locX , myTank.locY , null);

        //Draw enemy tank
        try {
            image2 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\tank_dark.png"));
        } catch (IOException e) {
            e.printStackTrace();
        }
        g2d.drawImage(image2 , enemyTank.locX+30 , enemyTank.locY+30 , null);

        //Draw bullet
        if (bullets.size() != 0) {
            for (BulletState bullet : bullets) {
                if (System.currentTimeMillis() - bullet.getCreateTime() > 4000) {
                  //  bullets.remove(bullet);
                    continue;
                }
                bullet.update();
                g2d.setColor(Color.GRAY);
                g2d.fillOval(bullet.locX, bullet.locY, bullet.diam, bullet.diam);
            }
        }


        // Print FPS info
        long currentRender = System.currentTimeMillis();
        if (lastRender > 0) {
            fpsHistory.add(1000.0f / (currentRender - lastRender));
            if (fpsHistory.size() > 100) {
                fpsHistory.remove(0); // remove oldest
            }
            float avg = 0.0f;
            for (float fps : fpsHistory) {
                avg += fps;
            }
            avg /= fpsHistory.size();
            String str = String.format("Average FPS = %.1f , Last Interval = %d ms",
                    avg, (currentRender - lastRender));
            g2d.setColor(Color.CYAN);
            g2d.setFont(g2d.getFont().deriveFont(18.0f));
            int strWidth = g2d.getFontMetrics().stringWidth(str);
            int strHeight = g2d.getFontMetrics().getHeight();
            g2d.drawString(str, (GAME_WIDTH - strWidth) / 2, strHeight + 50);
        }
        lastRender = (int) currentRender;
        // Print user guide
        String userGuide
                = "Use the MOUSE or ARROW KEYS to move the TANK. "
                + "Press ESCAPE to end the game.";
        g2d.setFont(g2d.getFont().deriveFont(18.0f));
        g2d.drawString(userGuide, 10, GAME_HEIGHT - 10);
        // Draw GAME OVER
        if (myTank.gameOver) {
            String str = "GAME OVER";
            g2d.setColor(Color.WHITE);
            g2d.setFont(g2d.getFont().deriveFont(Font.BOLD).deriveFont(64.0f));
            int strWidth = g2d.getFontMetrics().stringWidth(str);
            g2d.drawString(str, (GAME_WIDTH - strWidth) / 2, GAME_HEIGHT / 2);
        }
    }
}
